@@ -16,13 +16,11 @@
 """bibliography processor
 """
 
-import locale
-
 
 def make_bibliography(aux_filename,
         bib_format=None,
-        bib_encoding=locale.getpreferredencoding(),
-        output_encoding=locale.getpreferredencoding(),
+        bib_encoding=None,
+        output_encoding=None,
         **kwargs
         ):
     """This functions extracts all nessessary information from .aux file
