@@ -16,33 +16,32 @@
 """BibTeX unnamed stack language interpreter and related stuff
 """
 
-import locale
-
 
 def make_bibliography(aux_filename,
         bib_format=None,
-        bib_encoding=locale.getpreferredencoding(),
-        output_encoding=locale.getpreferredencoding(),
-        bst_encoding=locale.getpreferredencoding(),
+        bib_encoding=None,
+        output_encoding=None,
+        bst_encoding=None,
+        min_crossrefs=2,
         **kwargs
     ):
 
-    import codecs
     from os import path
 
+    import pybtex.io
     from pybtex.bibtex import bst
     from pybtex.bibtex.interpreter import Interpreter
     from pybtex import auxfile
-    from pybtex.bibtex.kpathsea import kpsewhich
 
 
     if bib_format is None:
         from pybtex.database.input import bibtex as bib_format
     aux_data = auxfile.parse_file(aux_filename, output_encoding)
-    bst_filename = kpsewhich(aux_data.style + path.extsep + 'bst')
+    bst_filename = aux_data.style + path.extsep + 'bst'
     bst_script = bst.parse_file(bst_filename, bst_encoding)
     base_filename = path.splitext(aux_filename)[0]
     bbl_filename = base_filename + path.extsep + 'bbl'
     bib_filenames = [filename + path.extsep + bib_format.file_extension for filename in aux_data.data]
-    bbl_file = codecs.open(bbl_filename, 'w', encoding=output_encoding)
-    Interpreter(bib_format).run(bst_script, aux_data.citations, bib_filenames, bbl_file)
+    bbl_file = pybtex.io.open_unicode(bbl_filename, 'w', encoding=output_encoding)
+    interpreter = Interpreter(bib_format, bib_encoding)
+    interpreter.run(bst_script, aux_data.citations, bib_filenames, bbl_file, min_crossrefs=min_crossrefs)
