@@ -18,10 +18,10 @@
 CAUTION: functions should PUSH results, not RETURN
 """
 
+import pybtex.io
 from pybtex.bibtex.exceptions import BibTeXError
-from pybtex.bibtex.utils import split_name_list
+from pybtex.bibtex import utils
 from pybtex.core import Person
-from pybtex.bibtex.utils import bibtex_len, bibtex_prefix, bibtex_purify, bibtex_substring
 from pybtex.bibtex.names import format as format_bibtex_name
 
 class Builtin(object):
@@ -105,32 +105,17 @@ def call_type(i):
 
 @builtin('change.case$')
 def change_case(i):
-    def title(s):
-        l = []
-        start = True
-        for pos, char in enumerate(s):
-            if char == ':':
-                try:
-                    start = s[pos+1].isspace()
-                except IndexError:
-                    pass
-                start = True
-            if start:
-                l.append(char)
-                start = False
-            else:
-                l.append(char.lower())
-        return(''.join(l))
 
     mode = i.pop()
-    s = i.pop()
-    if mode == 'l':
-        s = s.lower()
-    elif mode == 'u':
-        s = s.upper()
-    elif mode == 't':
-        s = title(s)
-    i.push(s)
+    string = i.pop()
+
+    if not mode:
+        raise BibTeXError('empty mode string passed to change.case$')
+    mode_letter = mode[0].lower()
+    if not mode_letter in ('l', 'u', 't'):
+        raise BibTeXError('incorrect change.case$ mode: %s' % mode)
+
+    i.push(utils.change_case(string, mode_letter))
 
 @builtin('chr.to.int$')
 def chr_to_int(i):
@@ -165,7 +150,7 @@ def format_name(i):
     format = i.pop()
     n = i.pop()
     names = i.pop()
-    name = split_name_list(names)[n - 1]
+    name = utils.split_name_list(names)[n - 1]
     i.push(format_bibtex_name(name, format))
 
 @builtin('if$')
@@ -201,14 +186,12 @@ def missing(i):
 
 @builtin('newline$')
 def newline(i):
-    # FIXME bibtex does some automatic line breaking
-    # needs more investigation
     i.newline()
 
 @builtin('num.names$')
 def num_names(i):
     names = i.pop()
-    i.push(len(split_name_list(names)))
+    i.push(len(utils.split_name_list(names)))
 
 @builtin('pop$')
 def pop(i):
@@ -221,7 +204,7 @@ def preamble(i):
 @builtin('purify$')
 def purify(i):
     s = i.pop()
-    i.push(bibtex_purify(s))
+    i.push(utils.bibtex_purify(s))
 
 @builtin('quote$')
 def quote(i):
@@ -236,15 +219,12 @@ def substring(i):
     length = i.pop()
     start = i.pop()
     string = i.pop()
-    i.push(bibtex_substring(string, start, length))
+    i.push(utils.bibtex_substring(string, start, length))
 
 @builtin('stack$')
 def stack(i):
-    try:
-        while True:
-            print i.pop()
-    except IndexError:
-        pass
+    while i.stack:
+        print >>pybtex.io.stdout, i.pop()
 
 @builtin('swap$')
 def swap(i):
@@ -256,18 +236,17 @@ def swap(i):
 @builtin('text.length$')
 def text_length(i):
     s = i.pop()
-    i.push(bibtex_len(s))
+    i.push(utils.bibtex_len(s))
 
 @builtin('text.prefix$')
 def text_prefix(i):
     l = i.pop()
     s = i.pop()
-    i.push(bibtex_prefix(s, l))
+    i.push(utils.bibtex_prefix(s, l))
 
 @builtin('top$')
 def top(i):
-    #FIXME encoding
-    print (unicode(i.pop()).encode('UTF-8'))
+    print >>pybtex.io.stdout, i.pop()
 
 @builtin('type$')
 def type_(i):
@@ -275,9 +254,8 @@ def type_(i):
 
 @builtin('warning$')
 def warning(i):
-    #FIXME stub
     msg = i.pop()
-    print 'Warning--' + msg
+    print >>pybtex.io.stderr, 'Warning--' + msg
 
 @builtin('while$')
 def while_(i):
@@ -293,7 +271,7 @@ def while_(i):
 def width(i):
     #FIXME need to investigate bibtex' source
     s = i.pop()
-    i.push(bibtex_len(s))
+    i.push(utils.bibtex_len(s))
 
 @builtin('write$')
 def write(i):
