@@ -110,6 +110,24 @@ class Field(object):
             return MissingField(self.name)
 
 
+class Crossref(Field):
+    def __init__(self, interpreter):
+        super(Crossref, self).__init__(interpreter, 'crossref')
+
+    def value(self):
+        entry = self.interpreter.current_entry
+        interpreter = self.interpreter
+        crossref = interpreter.current_entry.fields.get('crossref')
+        crossrefs = interpreter.bib_data.crossref_counts.get(crossref, 0)
+        if (
+            crossref in self.interpreter.citations
+            or crossrefs >= self.interpreter.min_crossrefs
+        ):
+            return super(Crossref, self).value()
+        else:
+            return MissingField(self.name)
+
+
 class Identifier(Variable):
     value_type = basestring
     def execute(self, interpreter):
@@ -153,8 +171,9 @@ class FunctionLiteral(Function):
 
 
 class Interpreter(object):
-    def __init__(self, bib_format):
+    def __init__(self, bib_format, bib_encoding):
         self.bib_format = bib_format
+        self.bib_encoding = bib_encoding
         self.stack = []
         self.vars = dict(builtins)
         #FIXME is 10000 OK?
@@ -170,7 +189,10 @@ class Interpreter(object):
 #        print 'stack:', self.stack
 
     def pop(self):
-        value = self.stack.pop()
+        try:
+            value = self.stack.pop()
+        except IndexError:
+            raise BibTeXError('pop from empty stack')
 #        print 'pop <%s>' % value
         return value
 
@@ -191,11 +213,12 @@ class Interpreter(object):
         self.output_file.write('\n')
         self.output_buffer = []
 
-    def run(self, bst_script, citations, bib_files, bbl_file):
+    def run(self, bst_script, citations, bib_files, bbl_file, min_crossrefs):
         self.bst_script = iter(bst_script)
         self.citations = citations
         self.bib_files = bib_files
         self.output_file = bbl_file
+        self.min_crossrefs = min_crossrefs
 
         for i in self.bst_script:
             commandname = 'command_' + i
@@ -210,7 +233,7 @@ class Interpreter(object):
         for id in self.get_token():
             name = id.value()
             self.add_variable(name, Field(self, name))
-        self.add_variable('crossref', Field(self, 'crossref'))
+        self.add_variable('crossref', Crossref(self))
         for id in self.get_token():
             name = id.value()
             self.add_variable(name, EntryInteger(self, name))
@@ -248,19 +271,25 @@ class Interpreter(object):
         value = self.get_token()[0].value()
         self.macros[name] = value
 
-    def expand_wildcard_citations(self):
-        for citation in self.citations:
+    def expand_wildcard_citations(self, citations):
+        for citation in citations:
             if citation == '*':
                 for key in self.bib_data.entries.iterkeys():
                     yield key
             else:
                 yield citation
 
+    def add_crossreferenced_citations(self, citations):
+        citation_set = set(citations)
+        extra_citations = self.bib_data.get_extra_citations(self.min_crossrefs)
+        return citations + [citation for citation in extra_citations if citation not in citation_set]
+
     def command_read(self):
 #        print 'READ'
-        p = self.bib_format.Parser(macros=self.macros, person_fields=[])
+        p = self.bib_format.Parser(encoding=self.bib_encoding, macros=self.macros, person_fields=[])
         self.bib_data = p.parse_files(self.bib_files)
-        self.citations = list(self.expand_wildcard_citations())
+        citations = list(self.expand_wildcard_citations(self.citations))
+        self.citations = self.add_crossreferenced_citations(citations)
 #        for k, v in self.bib_data.iteritems():
 #            print k
 #            for field, value in v.fields.iteritems():
