@@ -15,6 +15,7 @@
 
 import re
 
+from pybtex.bibtex.exceptions import BibTeXError
 
 whitespace_re = re.compile('(\s)')
 
@@ -50,6 +51,117 @@ def wrap(string, width=79):
     return '\n'.join(wrap_chunks(chunks, width))
 
 
+class BibTeXString(object):
+    def __init__(self, chars, level=0):
+        self.level = level
+        self.is_closed = False
+        self.contents = list(self.find_closing_brace(iter(chars)))
+
+    def find_closing_brace(self, chars):
+        for char in chars:
+            if char == '{':
+                yield BibTeXString(chars, self.level + 1)
+            elif char == '}' and self.level > 0:
+                self.is_closed = True
+                return
+            else:
+                yield char
+
+    def is_special_char(self):
+        return self.level == 1 and self.contents and self.contents[0] == '\\'
+
+    def traverse(self, open=None, f=lambda char, string: char, close=None):
+        if open is not None and self.level > 0:
+            yield open(self)
+
+        for child in self.contents:
+            if hasattr(child, 'traverse'):
+                if child.is_special_char():
+                    yield open(child)
+                    yield f(child.inner_string(), child)
+                    yield close(child)
+                else:
+                    for result in child.traverse(open, f, close):
+                        yield result
+            else:
+                yield f(child, self)
+
+        if close is not None and self.level > 0 and self.is_closed:
+            yield close(self)
+
+    def __unicode__(self):
+        return ''.join(self.traverse(open=lambda string: '{', close=lambda string: '}'))
+
+    def inner_string(self):
+        return ''.join(unicode(child) for child in self.contents)
+
+
+def change_case(string, mode):
+    r"""
+    >>> print change_case('aBcD', 'l')
+    abcd
+    >>> print change_case('aBcD', 'u')
+    ABCD
+    >>> print change_case('ABcD', 't')
+    Abcd
+    >>> print change_case(r'The {\TeX book \noop}', 'u')
+    THE {\TeX BOOK \noop}
+    >>> print change_case('And Now: BOOO!!!', 't')
+    And now: Booo!!!
+    >>> print change_case('And {Now: BOOO!!!}', 't')
+    And {Now: BOOO!!!}
+    >>> print change_case('And {Now: {BOOO}!!!}', 'l')
+    and {Now: {BOOO}!!!}
+    >>> print change_case('And {\Now: BOOO!!!}', 't')
+    And {\Now: booo!!!}
+    >>> print change_case('And {\Now: {BOOO}!!!}', 'l')
+    and {\Now: {booo}!!!}
+    """
+
+    def title(char, state):
+        if state == 'start':
+            return char
+        else:
+            return char.lower()
+
+    lower = lambda char, state: char.lower()
+    upper = lambda char, state: char.upper()
+
+    convert_level0 = {'l': lower, 'u': upper, 't': title}[mode]
+    convert_level1 = {'l': lower, 'u': upper, 't': lower}[mode]
+
+    def convert_special_char(special_char):
+        # FIXME BibTeX treats some accented and foreign characterss specially
+        def convert_words(words):
+            for word in words:
+                if word.startswith('\\'):
+                    yield word
+                else:
+                    yield convert_level1(word, 'normal')
+
+        return ' '.join(convert_words(special_char.split(' ')))
+
+    def change_case_iter(string, mode):
+        state = 'start'
+        for char, brace_level in scan_bibtex_string(string):
+            if brace_level == 0:
+                yield convert_level0(char, state)
+                if char == ':':
+                    state = 'after colon'
+                elif char.isspace() and state == 'after colon':
+                    state = 'start'
+                else:
+                    state = 'normal'
+            else:
+                if brace_level == 1 and char.startswith('\\'):
+                    yield convert_special_char(char)
+                else:
+                    yield char
+                state = 'normal'
+
+    return ''.join(change_case_iter(string, mode))
+
+
 def bibtex_substring(string, start, length):
     """
     Return a substring of the given length, starting from the given position.
@@ -184,6 +296,7 @@ def bibtex_purify(string):
     abc
     """
 
+    # FIXME BibTeX treats some accented and foreign characterss specially
     def purify_iter():
         for char, brace_level in scan_bibtex_string(string):
             if brace_level == 1 and char.startswith('\\'):
@@ -195,65 +308,17 @@ def bibtex_purify(string):
     return ''.join(purify_iter())
 
 
-class LookAheadIterator(object):
-    def __init__(self, seq):
-        self.iterable = iter(seq)
-        self.buffer = []
-
-    def __iter__(self):
-        return self
-
-    def peek(self):
-        if self.buffer:
-            return self.buffer[0]
-        else:
-            try:
-                self.buffer.append(self.iterable.next())
-            except StopIteration:
-                return None
-            else:
-                return self.buffer[0]
-
-    def poke(self, token):
-        self.buffer.insert(0, token)
-
-    def next(self):
-        if self.buffer:
-            return self.buffer.pop()
-        else:
-            return self.iterable.next()
-
-
 def scan_bibtex_string(string):
     """ Yield (char, brace_level) tuples.
 
     "Special characters", as in bibtex_len, are treated as a single character
 
     """
-
-    def scan(chars, level=0):
-        results = find_closing_brace(chars, level)
-        if level == 1 and chars.peek() == '\\':
-            return group(results, level)
-        else:
-            return results
-
-    def group(results, level):
-        yield ''.join(char for char, _ in results), level
-    
-    def find_closing_brace(chars, level):
-        for char in chars:
-            if char == '{':
-                yield char, level + 1
-                for item in scan(chars, level + 1):
-                    yield item
-            elif char == '}' and level > 0:
-                chars.poke(char)
-                return
-            else:
-                yield char, level
-    
-    return scan(LookAheadIterator(string))
+    return BibTeXString(string).traverse(
+        open=lambda string: ('{', string.level),
+        f=lambda char, string: (char, string.level),
+        close=lambda string: ('}', string.level - 1),
+    )
 
 
 def split_name_list(string):
