@@ -15,9 +15,9 @@
 
 from __future__ import with_statement
 
-import locale
 from os import path
 
+import pybtex.io
 from pybtex.database import BibliographyData
 
 default = 'bib'
@@ -25,15 +25,13 @@ filetypes = {'bib' : 'bibtex'}
 
 class ParserBase:
     def __init__(self, encoding=None, **kwargs):
-        if encoding is None:
-            encoding = locale.getpreferredencoding()
         self.encoding = encoding
         self.data = BibliographyData()
 
     def parse_file(self, filename, fileext=None):
         if fileext is not None:
             filename = filename + path.extsep + fileext
-        with open(filename, 'r') as f:
+        with pybtex.io.open_plain(filename, 'r') as f:
             self.parse_stream(f)
         return self.data
 
@@ -41,3 +39,6 @@ class ParserBase:
         for filename in filenames:
             self.parse_file(filename, fileext)
         return self.data
+
+    def parse_stream(self, stream):
+        raise NotImplementedError
