@@ -15,13 +15,17 @@
 
 from __future__ import with_statement
 
+import pybtex.io
+
 class WriterBase:
     def __init__(self, encoding = None):
         if not encoding:
-            import locale
-            encoding = locale.getpreferredencoding()
+            encoding = pybtex.io.get_default_encoding()
         self.encoding = encoding
 
     def write_file(self, bib_data, filename):
-        with open(filename, 'w') as stream:
+        with pybtex.io.open_plain(filename, 'w') as stream:
             self.write_stream(bib_data, stream)
+
+    def write_stream(self, bib_data, stream):
+        raise NotImplementedError
