@@ -13,8 +13,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import locale
-import codecs
 from pybtex.core import FormattedEntry
 from pybtex.style.template import node, join
 from pybtex.richtext import Symbol, Text
