@@ -1,3 +1,18 @@
+# Copyright (C) 2009  Andrey Golovizin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 from unittest import TestCase
 from pkg_resources import resource_stream
 import yaml
@@ -16,7 +31,8 @@ class DatabaseIOTest(TestCase):
         stream = StringIO()
         writer.write_stream(self.reference_data, stream)
         stream.seek(0)
-        loaded_data = parser.parse_stream(stream)
+        parser.parse_stream(stream)
+        loaded_data = parser.data
         self.assertEqual(loaded_data, self.reference_data)
 
     def test_bibtex_inpub(self):
